@@ -29,6 +29,7 @@
 #include "PluginView.h"
 
 #include "BitmapImage.h"
+#include "BitmapInfo.h"
 #include "BridgeJSC.h"
 #include "Chrome.h"
 #include "ChromeClient.h"
@@ -66,15 +67,11 @@
 #include "c_instance.h"
 #include "npruntime_impl.h"
 #include "runtime_root.h"
+#include <runtime/JSCJSValue.h>
 #include <runtime/JSLock.h>
-#include <runtime/JSValue.h>
 #include <wtf/ASCIICType.h>
 #include <wtf/text/WTFString.h>
 
-#if !PLATFORM(WX)
-#include "BitmapInfo.h"
-#endif
-
 #if OS(WINCE)
 #undef LOG_NPERROR
 #define LOG_NPERROR(x)
@@ -97,11 +94,6 @@
 #include <QWindow>
 #endif
 
-#if PLATFORM(WX)
-#include <wx/defs.h>
-#include <wx/window.h>
-#endif
-
 static inline HWND windowHandleForPageClient(PlatformPageClient client)
 {
 #if PLATFORM(GTK)
@@ -116,10 +108,6 @@ static inline HWND windowHandleForPageClient(PlatformPageClient client)
     if (QWindow* window = client->ownerWindow())
         return reinterpret_cast<HWND>(window->winId());
     return 0;
-#elif PLATFORM(WX)
-    if (!client)
-        return 0;
-    return (HWND)client->GetHandle();
 #else
     return client;
 #endif
@@ -362,11 +350,10 @@ static inline IntPoint contentsToNativeWindow(FrameView* view, const IntPoint& p
 #if PLATFORM(QT)
     // Our web view's QWidget isn't necessarily a native window itself. Map the position
     // all the way up to the QWidget associated with the HWND returned as NPNVnetscapeWindow.
-    PlatformPageClient client = view->hostWindow()->platformPageClient();
-    return client->mapToOwnerWindow(view->contentsToWindow(point));
-#else
-    return view->contentsToWindow(point);
+    if (PlatformPageClient client = view->hostWindow()->platformPageClient())
+        return client->mapToOwnerWindow(view->contentsToWindow(point));
 #endif
+    return view->contentsToWindow(point);
 }
 
 static inline IntRect contentsToNativeWindow(FrameView* view, const IntRect& rect)
@@ -455,7 +442,7 @@ void PluginView::updatePluginWidget()
         return;
 
     ASSERT(parent()->isFrameView());
-    FrameView* frameView = static_cast<FrameView*>(parent());
+    FrameView* frameView = toFrameView(parent());
 
     IntRect oldWindowRect = m_windowRect;
     IntRect oldClipRect = m_clipRect;
@@ -551,7 +538,7 @@ bool PluginView::dispatchNPEvent(NPEvent& npEvent)
         shouldPop = true;
     }
 
-    JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonJSGlobalData());
+    JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonVM());
     setCallingPlugin(true);
     bool accepted = !m_plugin->pluginFuncs()->event(m_instance, &npEvent);
     setCallingPlugin(false);
@@ -576,7 +563,7 @@ void PluginView::paintIntoTransformedContext(HDC hdc)
 
     WINDOWPOS windowpos = { 0, 0, 0, 0, 0, 0, 0 };
 
-    IntRect r = contentsToNativeWindow(static_cast<FrameView*>(parent()), frameRect());
+    IntRect r = contentsToNativeWindow(toFrameView(parent()), frameRect());
 
     windowpos.x = r.x();
     windowpos.y = r.y();
@@ -604,12 +591,12 @@ void PluginView::paintIntoTransformedContext(HDC hdc)
 
 void PluginView::paintWindowedPluginIntoContext(GraphicsContext* context, const IntRect& rect)
 {
-#if !OS(WINCE)
+#if !USE(WINGDI)
     ASSERT(m_isWindowed);
     ASSERT(context->shouldIncludeChildWindows());
 
     ASSERT(parent()->isFrameView());
-    IntPoint locationInWindow = static_cast<FrameView*>(parent())->contentsToWindow(frameRect().location());
+    IntPoint locationInWindow = toFrameView(parent())->convertToContainingWindow(frameRect().location());
 
     LocalWindowsContext windowsContext(context, frameRect(), false);
 
@@ -654,7 +641,7 @@ void PluginView::paint(GraphicsContext* context, const IntRect& rect)
         setNPWindowRect(frameRect());
 
     if (m_isWindowed) {
-#if !OS(WINCE)
+#if !USE(WINGDI)
         if (context->shouldIncludeChildWindows())
             paintWindowedPluginIntoContext(context, rect);
 #endif
@@ -669,7 +656,7 @@ void PluginView::paint(GraphicsContext* context, const IntRect& rect)
     IntRect rectInWindow(rect);
     rectInWindow.intersect(frameRect());
 #else
-    IntRect rectInWindow = static_cast<FrameView*>(parent())->contentsToWindow(frameRect());
+    IntRect rectInWindow = toFrameView(parent())->contentsToWindow(frameRect());
 #endif
     LocalWindowsContext windowsContext(context, rectInWindow, m_isTransparent);
 
@@ -710,7 +697,7 @@ void PluginView::handleKeyboardEvent(KeyboardEvent* event)
     } else
         return;
 
-    JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonJSGlobalData());
+    JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonVM());
     if (dispatchNPEvent(npEvent))
         event->setDefaultHandled();
 }
@@ -725,7 +712,7 @@ void PluginView::handleMouseEvent(MouseEvent* event)
 
     NPEvent npEvent;
 
-    IntPoint p = contentsToNativeWindow(static_cast<FrameView*>(parent()), IntPoint(event->pageX(), event->pageY()));
+    IntPoint p = contentsToNativeWindow(toFrameView(parent()), IntPoint(event->pageX(), event->pageY()));
 
     npEvent.lParam = MAKELPARAM(p.x(), p.y());
     npEvent.wParam = 0;
@@ -780,17 +767,17 @@ void PluginView::handleMouseEvent(MouseEvent* event)
     } else
         return;
 
-    JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonJSGlobalData());
+    JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonVM());
     // FIXME: Consider back porting the http://webkit.org/b/58108 fix here.
     if (dispatchNPEvent(npEvent))
         event->setDefaultHandled();
 
-#if !PLATFORM(GTK) && !PLATFORM(QT) && !PLATFORM(WX) && !OS(WINCE)
+#if !PLATFORM(GTK) && !PLATFORM(QT) && !OS(WINCE)
     // Currently, Widget::setCursor is always called after this function in EventHandler.cpp
     // and since we don't want that we set ignoreNextSetCursor to true here to prevent that.
     ignoreNextSetCursor = true;
     if (Page* page = m_parentFrame->page())
-        page->chrome()->client()->setLastSetCursorToCurrentCursor();
+        page->chrome().client()->setLastSetCursorToCurrentCursor();
 #endif
 }
 
@@ -844,7 +831,7 @@ void PluginView::setNPWindowRect(const IntRect& rect)
         return;
 
 #if OS(WINCE)
-    IntRect r = static_cast<FrameView*>(parent())->contentsToWindow(rect);
+    IntRect r = toFrameView(parent())->contentsToWindow(rect);
     m_npWindow.x = r.x();
     m_npWindow.y = r.y();
 
@@ -859,7 +846,7 @@ void PluginView::setNPWindowRect(const IntRect& rect)
 # if PLATFORM(GTK) || PLATFORM(QT)
     IntPoint p = rect.location();
 # else
-    IntPoint p = static_cast<FrameView*>(parent())->contentsToWindow(rect.location());
+    IntPoint p = toFrameView(parent())->contentsToWindow(rect.location());
 # endif
     m_npWindow.x = p.x();
     m_npWindow.y = p.y();
@@ -874,7 +861,7 @@ void PluginView::setNPWindowRect(const IntRect& rect)
     m_npWindow.clipRect.top = 0;
 
     if (m_plugin->pluginFuncs()->setwindow) {
-        JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonJSGlobalData());
+        JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonVM());
         setCallingPlugin(true);
         m_plugin->pluginFuncs()->setwindow(m_instance, &m_npWindow);
         setCallingPlugin(false);
@@ -909,13 +896,13 @@ NPError PluginView::handlePostReadFile(Vector<char>& buffer, uint32_t len, const
 
     // Get file info
     WIN32_FILE_ATTRIBUTE_DATA attrs;
-    if (GetFileAttributesExW(filename.charactersWithNullTermination(), GetFileExInfoStandard, &attrs) == 0)
+    if (GetFileAttributesExW(filename.charactersWithNullTermination().data(), GetFileExInfoStandard, &attrs) == 0)
         return NPERR_FILE_NOT_FOUND;
 
     if (attrs.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
         return NPERR_FILE_NOT_FOUND;
 
-    HANDLE fileHandle = CreateFileW(filename.charactersWithNullTermination(), FILE_READ_DATA, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
+    HANDLE fileHandle = CreateFileW(filename.charactersWithNullTermination().data(), FILE_READ_DATA, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
     
     if (fileHandle == INVALID_HANDLE_VALUE)
         return NPERR_FILE_NOT_FOUND;
@@ -1036,7 +1023,7 @@ bool PluginView::platformStart()
         HWND window = ::CreateWindowEx(0, kWebPluginViewdowClassName, 0, flags,
                                        0, 0, 0, 0, parentWindowHandle, 0, WebCore::instanceHandle(), 0);
 
-#if OS(WINDOWS) && (PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(WX))
+#if OS(WINDOWS) && (PLATFORM(GTK) || PLATFORM(QT))
         m_window = window;
 #else
         setPlatformWidget(window);
@@ -1079,7 +1066,7 @@ void PluginView::platformDestroy()
 
 PassRefPtr<Image> PluginView::snapshot()
 {
-#if !PLATFORM(GTK) && !PLATFORM(WX) && !OS(WINCE)
+#if !PLATFORM(GTK) && !USE(WINGDI)
     OwnPtr<HDC> hdc = adoptPtr(CreateCompatibleDC(0));
 
     if (!m_isWindowed) {
@@ -1092,7 +1079,7 @@ PassRefPtr<Image> PluginView::snapshot()
         // Windowless plug-ins assume that they're drawing onto the view's DC.
         // Translate the context so that the plug-in draws at (0, 0).
         ASSERT(parent()->isFrameView());
-        IntPoint position = static_cast<FrameView*>(parent())->contentsToWindow(frameRect()).location();
+        IntPoint position = toFrameView(parent())->contentsToWindow(frameRect()).location();
         transform.eDx = -position.x();
         transform.eDy = -position.y();
         SetWorldTransform(hdc.get(), &transform);
