@@ -30,26 +30,38 @@
 #include "ContextMenuClientQt.h"
 #include "ContextMenuController.h"
 #if ENABLE(DEVICE_ORIENTATION)
-#include "DeviceMotionClientQt.h"
+#include "DeviceMotionClientMock.h"
+#include "DeviceMotionController.h"
 #include "DeviceOrientationClientMock.h"
+#include "DeviceOrientationController.h"
+#if HAVE(QTSENSORS)
+#include "DeviceMotionClientQt.h"
 #include "DeviceOrientationClientQt.h"
 #endif
+#endif
 #include "DocumentLoader.h"
 #include "DragClientQt.h"
 #include "DragController.h"
 #include "DragData.h"
 #include "DragSession.h"
+#include "Editor.h"
 #include "EditorClientQt.h"
+#include "EventHandler.h"
 #include "FocusController.h"
+#include "FrameLoadRequest.h"
+#include "FrameSelection.h"
 #include "FrameView.h"
 #if ENABLE(GEOLOCATION)
 #include "GeolocationClientMock.h"
-#include "GeolocationClientQt.h"
 #include "GeolocationController.h"
+#if HAVE(QTPOSITIONING)
+#include "GeolocationClientQt.h"
+#endif
 #endif
 #include "GeolocationPermissionClientQt.h"
 #include "HTMLFrameOwnerElement.h"
 #include "HTMLInputElement.h"
+#include "HTMLMediaElement.h"
 #include "HitTestResult.h"
 #include "InitWebCoreQt.h"
 #include "InspectorClientQt.h"
@@ -58,11 +70,13 @@
 #include "LocalizedStrings.h"
 #include "MIMETypeRegistry.h"
 #include "MemoryCache.h"
+#include "NetworkingContext.h"
 #include "NodeList.h"
 #include "NotificationPresenterClientQt.h"
 #include "PageGroup.h"
 #include "Pasteboard.h"
 #include "PlatformKeyboardEvent.h"
+#include "PlatformMouseEvent.h"
 #include "PlatformTouchEvent.h"
 #include "PlatformWheelEvent.h"
 #include "PluginDatabase.h"
@@ -76,6 +90,7 @@
 #include "Settings.h"
 #include "UndoStepQt.h"
 #include "UserAgentQt.h"
+#include "UserGestureIndicator.h"
 #include "WebEventConversion.h"
 #include "WebKitVersion.h"
 #include "WindowFeatures.h"
@@ -136,6 +151,40 @@ static inline Qt::DropAction dragOpToDropAction(unsigned actions)
     return result;
 }
 
+static inline WebCore::PageVisibilityState webPageVisibilityStateToWebCoreVisibilityState(QWebPageAdapter::VisibilityState state)
+{
+    switch (state) {
+    case QWebPageAdapter::VisibilityStatePrerender:
+        return WebCore::PageVisibilityStatePrerender;
+    case QWebPageAdapter::VisibilityStateUnloaded:
+        return WebCore::PageVisibilityStateUnloaded;
+    case QWebPageAdapter::VisibilityStateVisible:
+        return WebCore::PageVisibilityStateVisible;
+    case QWebPageAdapter::VisibilityStateHidden:
+        return WebCore::PageVisibilityStateHidden;
+    default:
+        ASSERT(false);
+        return WebCore::PageVisibilityStateHidden;
+    }
+}
+
+static inline QWebPageAdapter::VisibilityState webCoreVisibilityStateToWebPageVisibilityState(WebCore::PageVisibilityState state)
+{
+    switch (state) {
+    case WebCore::PageVisibilityStatePrerender:
+        return QWebPageAdapter::VisibilityStatePrerender;
+    case WebCore::PageVisibilityStateUnloaded:
+        return QWebPageAdapter::VisibilityStateUnloaded;
+    case WebCore::PageVisibilityStateVisible:
+        return QWebPageAdapter::VisibilityStateVisible;
+    case WebCore::PageVisibilityStateHidden:
+        return QWebPageAdapter::VisibilityStateHidden;
+    default:
+        ASSERT(false);
+        return QWebPageAdapter::VisibilityStateHidden;
+    }
+}
+
 static WebCore::FrameLoadRequest frameLoadRequest(const QUrl &url, WebCore::Frame *frame)
 {
     return WebCore::FrameLoadRequest(frame->document()->securityOrigin(),
@@ -148,9 +197,9 @@ static void openNewWindow(const QUrl& url, Frame* frame)
         WindowFeatures features;
         NavigationAction action;
         FrameLoadRequest request = frameLoadRequest(url, frame);
-        if (Page* newPage = oldPage->chrome()->createWindow(frame, request, features, action)) {
+        if (Page* newPage = oldPage->chrome().createWindow(frame, request, features, action)) {
             newPage->mainFrame()->loader()->loadFrameRequest(request, false, false, 0, 0, MaybeSendReferrer);
-            newPage->chrome()->show();
+            newPage->chrome().show();
         }
     }
 }
@@ -162,6 +211,7 @@ QWebPageAdapter::QWebPageAdapter()
     , forwardUnsupportedContent(false)
     , insideOpenCall(false)
     , clickCausedFocus(false)
+    , m_useNativeVirtualKeyAsDOMKey(false)
     , m_totalBytes(0)
     , m_bytesReceived()
     , networkManager(0)
@@ -172,7 +222,7 @@ QWebPageAdapter::QWebPageAdapter()
 void QWebPageAdapter::initializeWebCorePage()
 {
 #if ENABLE(GEOLOCATION) || ENABLE(DEVICE_ORIENTATION)
-    bool useMock = QWebPageAdapter::drtRun;
+    const bool useMock = QWebPageAdapter::drtRun;
 #endif
     Page::PageClients pageClients;
     pageClients.chromeClient = new ChromeClientQt(this);
@@ -181,21 +231,34 @@ void QWebPageAdapter::initializeWebCorePage()
     pageClients.dragClient = new DragClientQt(pageClients.chromeClient);
     pageClients.inspectorClient = new InspectorClientQt(this);
     page = new Page(pageClients);
+
 #if ENABLE(GEOLOCATION)
     if (useMock) {
         // In case running in DumpRenderTree mode set the controller to mock provider.
         GeolocationClientMock* mock = new GeolocationClientMock;
         WebCore::provideGeolocationTo(page, mock);
         mock->setController(WebCore::GeolocationController::from(page));
-    } else
+    }
+#if HAVE(QTPOSITIONING)
+    else
         WebCore::provideGeolocationTo(page, new GeolocationClientQt(this));
 #endif
+#endif
+
 #if ENABLE(DEVICE_ORIENTATION)
-    if (useMock)
-        WebCore::provideDeviceOrientationTo(page, new DeviceOrientationClientMock);
-    else
+    if (useMock) {
+        DeviceOrientationClientMock* mockOrientationClient = new DeviceOrientationClientMock;
+        WebCore::provideDeviceOrientationTo(page, mockOrientationClient);
+
+        DeviceMotionClientMock* mockMotionClient= new DeviceMotionClientMock;
+        WebCore::provideDeviceMotionTo(page, mockMotionClient);
+    }
+#if HAVE(QTSENSORS)
+    else {
         WebCore::provideDeviceOrientationTo(page, new DeviceOrientationClientQt);
-    WebCore::provideDeviceMotionTo(page, new DeviceMotionClientQt);
+        WebCore::provideDeviceMotionTo(page, new DeviceMotionClientQt);
+    }
+#endif
 #endif
 
     // By default each page is put into their own unique page group, which affects popup windows
@@ -240,7 +303,7 @@ void QWebPageAdapter::deletePage()
 
 QWebPageAdapter* QWebPageAdapter::kit(Page* page)
 {
-    return static_cast<ChromeClientQt*>(page->chrome()->client())->m_webPage;
+    return static_cast<ChromeClientQt*>(page->chrome().client())->m_webPage;
 }
 
 ViewportArguments QWebPageAdapter::viewportArguments() const
@@ -254,6 +317,26 @@ void QWebPageAdapter::registerUndoStep(WTF::PassRefPtr<WebCore::UndoStep> step)
     createUndoStep(QSharedPointer<UndoStepQt>(new UndoStepQt(step)));
 }
 
+void QWebPageAdapter::setVisibilityState(VisibilityState state)
+{
+#if ENABLE(PAGE_VISIBILITY_API)
+    if (!page)
+        return;
+    page->setVisibilityState(webPageVisibilityStateToWebCoreVisibilityState(state), false);
+#else
+    Q_UNUSED(state);
+#endif
+}
+
+QWebPageAdapter::VisibilityState QWebPageAdapter::visibilityState() const
+{
+#if ENABLE(PAGE_VISIBILITY_API)
+    return webCoreVisibilityStateToWebPageVisibilityState(page->visibilityState());
+#else
+    return QWebPageAdapter::VisibilityStateVisible;
+#endif
+}
+
 void QWebPageAdapter::setNetworkAccessManager(QNetworkAccessManager *manager)
 {
     if (manager == networkManager)
@@ -283,12 +366,12 @@ QString QWebPageAdapter::selectedText() const
     Frame* frame = page->focusController()->focusedOrMainFrame();
     if (frame->selection()->selection().selectionType() == VisibleSelection::NoSelection)
         return QString();
-    return frame->editor()->selectedText();
+    return frame->editor().selectedText();
 }
 
 QString QWebPageAdapter::selectedHtml() const
 {
-    return page->focusController()->focusedOrMainFrame()->editor()->selectedRange()->toHTML();
+    return page->focusController()->focusedOrMainFrame()->editor().selectedRange()->toHTML();
 }
 
 bool QWebPageAdapter::isContentEditable() const
@@ -303,7 +386,7 @@ void QWebPageAdapter::setContentEditable(bool editable)
 
     Frame* frame = mainFrameAdapter()->frame;
     if (editable) {
-        frame->editor()->applyEditingStyleToBodyElement();
+        frame->editor().applyEditingStyleToBodyElement();
         // FIXME: mac port calls this if there is no selectedDOMRange
         // frame->setSelectionFromNone();
     }
@@ -312,16 +395,32 @@ void QWebPageAdapter::setContentEditable(bool editable)
 
 bool QWebPageAdapter::findText(const QString& subString, FindFlag options)
 {
-    ::TextCaseSensitivity caseSensitivity = ::TextCaseInsensitive;
-    if (options & FindCaseSensitively)
-        caseSensitivity = ::TextCaseSensitive;
+    ::WebCore::FindOptions webCoreFindOptions = 0;
+
+    if (!(options & FindCaseSensitively))
+        webCoreFindOptions |= WebCore::CaseInsensitive;
+
+    if (options & FindBackward)
+        webCoreFindOptions |= WebCore::Backwards;
+
+    if (options & FindWrapsAroundDocument)
+        webCoreFindOptions |= WebCore::WrapAround;
+
+    if (options & FindAtWordBeginningsOnly)
+        webCoreFindOptions |= WebCore::AtWordStarts;
+
+    if (options & TreatMedialCapitalAsWordBeginning)
+        webCoreFindOptions |= WebCore::TreatMedialCapitalAsWordStart;
+
+    if (options & FindBeginsInSelection)
+        webCoreFindOptions |= WebCore::StartInSelection;
 
     if (options & HighlightAllOccurrences) {
         if (subString.isEmpty()) {
             page->unmarkAllTextMatches();
             return true;
         }
-        return page->markAllMatchesForText(subString, caseSensitivity, true, 0);
+        return page->markAllMatchesForText(subString, webCoreFindOptions, /*shouldHighlight*/ true, /*limit*/ 0);
     }
 
     if (subString.isEmpty()) {
@@ -332,17 +431,13 @@ bool QWebPageAdapter::findText(const QString& subString, FindFlag options)
             frame = frame->tree()->traverseNextWithWrap(false);
         }
     }
-    ::FindDirection direction = ::FindDirectionForward;
-    if (options & FindBackward)
-        direction = ::FindDirectionBackward;
 
-    const bool shouldWrap = options & FindWrapsAroundDocument;
-
-    return page->findString(subString, caseSensitivity, direction, shouldWrap);
+    return page->findString(subString, webCoreFindOptions);
 }
 
 void QWebPageAdapter::adjustPointForClicking(QMouseEvent* ev)
 {
+#if ENABLE(TOUCH_ADJUSTMENT)
     QtPlatformPlugin platformPlugin;
     OwnPtr<QWebTouchModifier> touchModifier = platformPlugin.createTouchModifier();
     if (!touchModifier)
@@ -358,130 +453,27 @@ void QWebPageAdapter::adjustPointForClicking(QMouseEvent* ev)
     if (!topPadding && !rightPadding && !bottomPadding && !leftPadding)
         return;
 
-    Document* startingDocument = page->mainFrame()->document();
-    if (!startingDocument)
+    FrameView* view = page->mainFrame()->view();
+    ASSERT(view);
+    if (view->scrollbarAtPoint(ev->pos()))
         return;
 
-    IntPoint originalPoint(ev->pos());
-    TouchAdjuster touchAdjuster(topPadding, rightPadding, bottomPadding, leftPadding);
-    IntPoint adjustedPoint = touchAdjuster.findCandidatePointForTouch(originalPoint, startingDocument);
-    if (adjustedPoint == IntPoint::zero())
+    EventHandler* eventHandler = page->mainFrame()->eventHandler();
+    ASSERT(eventHandler);
+
+    IntRect touchRect(ev->pos().x() - leftPadding, ev->pos().y() - topPadding, leftPadding + rightPadding, topPadding + bottomPadding);
+    IntPoint adjustedPoint;
+    Node* adjustedNode;
+    bool foundClickableNode = eventHandler->bestClickableNodeForTouchPoint(touchRect.center(), touchRect.size(), adjustedPoint, adjustedNode);
+    if (!foundClickableNode)
         return;
+
     QMouseEvent* ret = new QMouseEvent(ev->type(), QPoint(adjustedPoint), ev->globalPos(), ev->button(), ev->buttons(), ev->modifiers());
     delete ev;
     ev = ret;
-}
-
-static bool hasMouseListener(Element* element)
-{
-    ASSERT(element);
-    return element->hasEventListeners(eventNames().clickEvent)
-        || element->hasEventListeners(eventNames().mousedownEvent)
-        || element->hasEventListeners(eventNames().mouseupEvent);
-}
-
-static bool isClickableElement(Element* element, PassRefPtr<NodeList> prpList)
-{
-    ASSERT(element);
-    RefPtr<NodeList> list = prpList;
-    bool isClickable = hasMouseListener(element);
-    if (!isClickable && list) {
-        Element* parent = element->parentElement();
-        unsigned count = list->length();
-        for (unsigned i = 0; i < count && parent; i++) {
-            if (list->item(i) != parent)
-                continue;
-
-            isClickable = hasMouseListener(parent);
-            if (isClickable)
-                break;
-
-            parent = parent->parentElement();
-        }
-    }
-
-    ExceptionCode ec = 0;
-    return isClickable
-        || element->webkitMatchesSelector("a,*:link,*:visited,*[role=button],button,input,select,label", ec)
-        || CSSComputedStyleDeclaration::create(element)->getPropertyValue(cssPropertyID("cursor")) == "pointer";
-}
-
-static bool isValidFrameOwner(Element* element)
-{
-    ASSERT(element);
-    return element->isFrameOwnerElement() && static_cast<HTMLFrameOwnerElement*>(element)->contentFrame();
-}
-
-QWebPageAdapter::TouchAdjuster::TouchAdjuster(unsigned topPadding, unsigned rightPadding, unsigned bottomPadding, unsigned leftPadding)
-    : m_topPadding(topPadding)
-    , m_rightPadding(rightPadding)
-    , m_bottomPadding(bottomPadding)
-    , m_leftPadding(leftPadding)
-{
-}
-
-IntPoint QWebPageAdapter::TouchAdjuster::findCandidatePointForTouch(const IntPoint& touchPoint, Document* document) const
-{
-    if (!document)
-        return IntPoint();
-
-    int x = touchPoint.x();
-    int y = touchPoint.y();
-
-    RefPtr<NodeList> intersectedNodes = document->nodesFromRect(x, y, m_topPadding, m_rightPadding, m_bottomPadding, m_leftPadding, false /*ignoreClipping*/, false /*allowShadowContent*/);
-    if (!intersectedNodes)
-        return IntPoint();
-
-    Element* closestClickableElement = 0;
-    IntRect largestIntersectionRect;
-    FrameView* view = document->frame()->view();
-
-    // Touch rect in contents coordinates.
-    IntRect touchRect(HitTestLocation::rectForPoint(view->windowToContents(IntPoint(x, y)), m_topPadding, m_rightPadding, m_bottomPadding, m_leftPadding));
-
-    // Iterate over the list of nodes hit looking for the one whose bounding area
-    // has largest intersection with the touch area (point + padding).
-    for (unsigned i = 0; i < intersectedNodes->length(); i++) {
-        Node* currentNode = intersectedNodes->item(i);
-
-        Element* currentElement = currentNode->isElementNode() ? toElement(currentNode) : 0;
-        if (!currentElement || (!isClickableElement(currentElement, 0) && !isValidFrameOwner(currentElement)))
-            continue;
-
-        IntRect currentElementBoundingRect = currentElement->pixelSnappedBoundingBox();
-        currentElementBoundingRect.intersect(touchRect);
-
-        if (currentElementBoundingRect.isEmpty())
-            continue;
-
-        int currentIntersectionRectArea = currentElementBoundingRect.width() * currentElementBoundingRect.height();
-        int largestIntersectionRectArea = largestIntersectionRect.width() * largestIntersectionRect.height();
-        if (currentIntersectionRectArea > largestIntersectionRectArea) {
-            closestClickableElement = currentElement;
-            largestIntersectionRect = currentElementBoundingRect;
-        }
-    }
-
-    if (largestIntersectionRect.isEmpty())
-        return IntPoint();
-
-    // Handle the case when user taps a inner frame. It is done in three steps:
-    // 1) Transform the original touch point to the inner document coordinates;
-    // 1) Call nodesFromRect for the inner document in case;
-    // 3) Re-add the inner frame offset (location) before passing the new clicking
-    //    position to WebCore.
-    if (closestClickableElement->isFrameOwnerElement()) {
-        // Adjust client coordinates' origin to be top left of inner frame viewport.
-        RefPtr<ClientRect> rect = closestClickableElement->getBoundingClientRect();
-        IntPoint newTouchPoint = touchPoint;
-        IntSize offset =  IntSize(rect->left(), rect->top());
-        newTouchPoint -= offset;
-
-        HTMLFrameOwnerElement* owner = static_cast<HTMLFrameOwnerElement*>(closestClickableElement);
-        Document* childDocument = owner->contentFrame()->document();
-        return findCandidatePointForTouch(newTouchPoint, childDocument);
-    }
-    return view->contentsToWindow(largestIntersectionRect).center();
+#else
+    Q_UNUSED(ev);
+#endif
 }
 
 void QWebPageAdapter::mouseMoveEvent(QMouseEvent* ev)
@@ -503,7 +495,7 @@ void QWebPageAdapter::mousePressEvent(QMouseEvent* ev)
     RefPtr<WebCore::Node> oldNode;
     Frame* focusedFrame = page->focusController()->focusedFrame();
     if (Document* focusedDocument = focusedFrame ? focusedFrame->document() : 0)
-        oldNode = focusedDocument->focusedNode();
+        oldNode = focusedDocument->focusedElement();
 
     if (tripleClickTimer.isActive()
         && (ev->pos() - tripleClick).manhattanLength() < qGuiApp->styleHints()->startDragDistance()) {
@@ -521,7 +513,7 @@ void QWebPageAdapter::mousePressEvent(QMouseEvent* ev)
     RefPtr<WebCore::Node> newNode;
     focusedFrame = page->focusController()->focusedFrame();
     if (Document* focusedDocument = focusedFrame ? focusedFrame->document() : 0)
-        newNode = focusedDocument->focusedNode();
+        newNode = focusedDocument->focusedElement();
 
     if (newNode && oldNode != newNode)
         clickCausedFocus = true;
@@ -581,10 +573,10 @@ void QWebPageAdapter::handleSoftwareInputPanel(Qt::MouseButton button, const QPo
         return;
 
     if (client && client->inputMethodEnabled()
-        && frame->document()->focusedNode()
+        && frame->document()->focusedElement()
             && button == Qt::LeftButton && qGuiApp->property("autoSipEnabled").toBool()) {
         if (!clickCausedFocus || requestSoftwareInputPanel()) {
-            HitTestResult result = frame->eventHandler()->hitTestResultAtPoint(frame->view()->windowToContents(pos), false);
+            HitTestResult result = frame->eventHandler()->hitTestResultAtPoint(frame->view()->windowToContents(pos));
             if (result.isContentEditable()) {
                 QEvent event(QEvent::RequestSoftwareInputPanel);
                 QGuiApplication::sendEvent(client->ownerWidget(), &event);
@@ -637,16 +629,16 @@ bool QWebPageAdapter::performDrag(const QMimeData *data, const QPoint &pos, Qt::
 void QWebPageAdapter::inputMethodEvent(QInputMethodEvent *ev)
 {
     WebCore::Frame *frame = page->focusController()->focusedOrMainFrame();
-    WebCore::Editor *editor = frame->editor();
+    WebCore::Editor &editor = frame->editor();
 
-    if (!editor->canEdit()) {
+    if (!editor.canEdit()) {
         ev->ignore();
         return;
     }
 
     Node* node = 0;
     if (frame->selection()->rootEditableElement())
-        node = frame->selection()->rootEditableElement()->shadowAncestorNode();
+        node = frame->selection()->rootEditableElement()->deprecatedShadowAncestorNode();
 
     Vector<CompositionUnderline> underlines;
     bool hasSelection = false;
@@ -675,17 +667,17 @@ void QWebPageAdapter::inputMethodEvent(QInputMethodEvent *ev)
             hasSelection = true;
             // A selection in the inputMethodEvent is always reflected in the visible text
             if (node) {
-                if (HTMLTextFormControlElement* textControl = toTextFormControl(node))
-                    textControl->setSelectionRange(qMin(a.start, (a.start + a.length)), qMax(a.start, (a.start + a.length)));
+                if (isHTMLTextFormControlElement(node))
+                    toHTMLTextFormControlElement(node)->setSelectionRange(qMin(a.start, (a.start + a.length)), qMax(a.start, (a.start + a.length)));
             }
 
             if (!ev->preeditString().isEmpty())
-                editor->setComposition(ev->preeditString(), underlines, qMin(a.start, (a.start + a.length)), qMax(a.start, (a.start + a.length)));
+                editor.setComposition(ev->preeditString(), underlines, qMin(a.start, (a.start + a.length)), qMax(a.start, (a.start + a.length)));
             else {
                 // If we are in the middle of a composition, an empty pre-edit string and a selection of zero
                 // cancels the current composition
-                if (editor->hasComposition() && !(a.start + a.length))
-                    editor->setComposition(QString(), underlines, 0, 0);
+                if (editor.hasComposition() && !(a.start + a.length))
+                    editor.setComposition(QString(), underlines, 0, 0);
             }
             break;
         }
@@ -697,19 +689,19 @@ void QWebPageAdapter::inputMethodEvent(QInputMethodEvent *ev)
     if (node && ev->replacementLength() > 0) {
         int cursorPos = frame->selection()->extent().offsetInContainerNode();
         int start = cursorPos + ev->replacementStart();
-        if (HTMLTextFormControlElement* textControl = toTextFormControl(node))
-            textControl->setSelectionRange(start, start + ev->replacementLength());
+        if (isHTMLTextFormControlElement(node))
+            toHTMLTextFormControlElement(node)->setSelectionRange(start, start + ev->replacementLength());
         // Commit regardless of whether commitString is empty, to get rid of selection.
-        editor->confirmComposition(ev->commitString());
+        editor.confirmComposition(ev->commitString());
     } else if (!ev->commitString().isEmpty()) {
-        if (editor->hasComposition())
-            editor->confirmComposition(ev->commitString());
+        if (editor.hasComposition())
+            editor.confirmComposition(ev->commitString());
         else
-            editor->insertText(ev->commitString(), 0);
+            editor.insertText(ev->commitString(), 0);
     } else if (!hasSelection && !ev->preeditString().isEmpty())
-        editor->setComposition(ev->preeditString(), underlines, 0, 0);
-    else if (ev->preeditString().isEmpty() && editor->hasComposition())
-        editor->confirmComposition(String());
+        editor.setComposition(ev->preeditString(), underlines, 0, 0);
+    else if (ev->preeditString().isEmpty() && editor.hasComposition())
+        editor.confirmComposition(String());
 
     ev->accept();
 }
@@ -720,13 +712,13 @@ QVariant QWebPageAdapter::inputMethodQuery(Qt::InputMethodQuery property) const
     if (!frame)
         return QVariant();
 
-    WebCore::Editor* editor = frame->editor();
+    WebCore::Editor& editor = frame->editor();
 
     RenderObject* renderer = 0;
     RenderTextControl* renderTextControl = 0;
 
     if (frame->selection()->rootEditableElement())
-        renderer = frame->selection()->rootEditableElement()->shadowAncestorNode()->renderer();
+        renderer = frame->selection()->rootEditableElement()->deprecatedShadowAncestorNode()->renderer();
 
     if (renderer && renderer->isTextControl())
         renderTextControl = toRenderTextControl(renderer);
@@ -748,14 +740,14 @@ QVariant QWebPageAdapter::inputMethodQuery(Qt::InputMethodQuery property) const
         return QVariant(QFont());
     }
     case Qt::ImCursorPosition: {
-        if (editor->hasComposition())
+        if (editor.hasComposition())
             return QVariant(frame->selection()->end().offsetInContainerNode());
         return QVariant(frame->selection()->extent().offsetInContainerNode());
     }
     case Qt::ImSurroundingText: {
         if (renderTextControl && renderTextControl->textFormControlElement()) {
             QString text = renderTextControl->textFormControlElement()->value();
-            RefPtr<Range> range = editor->compositionRange();
+            RefPtr<Range> range = editor.compositionRange();
             if (range)
                 text.remove(range->startPosition().offsetInContainerNode(), TextIterator::rangeLength(range.get()));
             return QVariant(text);
@@ -763,7 +755,7 @@ QVariant QWebPageAdapter::inputMethodQuery(Qt::InputMethodQuery property) const
         return QVariant();
     }
     case Qt::ImCurrentSelection: {
-        if (!editor->hasComposition() && renderTextControl && renderTextControl->textFormControlElement()) {
+        if (!editor.hasComposition() && renderTextControl && renderTextControl->textFormControlElement()) {
             int start = frame->selection()->start().offsetInContainerNode();
             int end = frame->selection()->end().offsetInContainerNode();
             if (end > start)
@@ -773,15 +765,15 @@ QVariant QWebPageAdapter::inputMethodQuery(Qt::InputMethodQuery property) const
 
     }
     case Qt::ImAnchorPosition: {
-        if (editor->hasComposition())
+        if (editor.hasComposition())
             return QVariant(frame->selection()->start().offsetInContainerNode());
         return QVariant(frame->selection()->base().offsetInContainerNode());
     }
     case Qt::ImMaximumTextLength: {
         if (frame->selection()->isContentEditable()) {
-            if (frame->document() && frame->document()->focusedNode()) {
-                if (frame->document()->focusedNode()->hasTagName(HTMLNames::inputTag)) {
-                    HTMLInputElement* inputElement = static_cast<HTMLInputElement*>(frame->document()->focusedNode());
+            if (frame->document() && frame->document()->focusedElement()) {
+                if (isHTMLInputElement(frame->document()->focusedElement())) {
+                    HTMLInputElement* inputElement = toHTMLInputElement(frame->document()->focusedElement());
                     return QVariant(inputElement->maxLength());
                 }
             }
@@ -854,6 +846,8 @@ void QWebPageAdapter::dynamicPropertyChangeEvent(QObject* obj, QDynamicPropertyC
     } else if (event->propertyName() == "_q_deadDecodedDataDeletionInterval") {
         double interval = obj->property("_q_deadDecodedDataDeletionInterval").toDouble();
         memoryCache()->setDeadDecodedDataDeletionInterval(interval);
+    }  else if (event->propertyName() == "_q_useNativeVirtualKeyAsDOMKey") {
+        m_useNativeVirtualKeyAsDOMKey = obj->property("_q_useNativeVirtualKeyAsDOMKey").toBool();
     }
 }
 
@@ -876,13 +870,13 @@ static QWebPageAdapter::MenuAction adapterActionForContextMenuAction(WebCore::Co
     return QWebPageAdapter::NoAction;
 }
 
-QList<MenuItem> descriptionForPlatformMenu(const QList<ContextMenuItem>* items, Page* page)
+QList<MenuItem> descriptionForPlatformMenu(const Vector<ContextMenuItem>& items, Page* page)
 {
     QList<MenuItem> itemDescriptions;
-    if (!items)
+    if (!items.size())
         return itemDescriptions;
-    for (int i = 0; i < items->count(); ++i) {
-        const ContextMenuItem &item = items->at(i);
+    for (int i = 0; i < items.size(); ++i) {
+        const ContextMenuItem &item = items.at(i);
         MenuItem description;
         switch (item.type()) {
         case WebCore::CheckableActionType: /* fall through */
@@ -893,14 +887,14 @@ QList<MenuItem> descriptionForPlatformMenu(const QList<ContextMenuItem>* items,
                 description.action = action;
                 ContextMenuItem it(item);
                 page->contextMenuController()->checkOrEnableIfNeeded(it);
-                PlatformMenuItemDescription desc = it.releasePlatformDescription();
-                if (desc.enabled)
+                if (it.enabled())
                     description.traits |= MenuItem::Enabled;
                 if (item.type() == WebCore::CheckableActionType) {
                     description.traits |= MenuItem::Checkable;
-                    if (desc.checked)
+                    if (it.checked())
                         description.traits |= MenuItem::Checked;
                 }
+                description.title = item.title();
             }
             break;
         }
@@ -909,8 +903,8 @@ QList<MenuItem> descriptionForPlatformMenu(const QList<ContextMenuItem>* items,
             break;
         case WebCore::SubmenuType: {
             description.type = MenuItem::SubMenu;
-            description.subMenu = descriptionForPlatformMenu(item.platformSubMenu(), page);
-            description.subMenuTitle = item.title();
+            description.subMenu = descriptionForPlatformMenu(item.subMenuItems(), page);
+            description.title = item.title();
             // Don't append empty submenu descriptions.
             if (description.subMenu.isEmpty())
                 continue;
@@ -926,7 +920,7 @@ QWebHitTestResultPrivate* QWebPageAdapter::updatePositionDependentMenuActions(co
 {
     ASSERT(visitedWebActions);
     WebCore::Frame* focusedFrame = page->focusController()->focusedOrMainFrame();
-    HitTestResult result = focusedFrame->eventHandler()->hitTestResultAtPoint(focusedFrame->view()->windowToContents(pos), /*allowShadowContent*/ false);
+    HitTestResult result = focusedFrame->eventHandler()->hitTestResultAtPoint(focusedFrame->view()->windowToContents(pos));
     page->contextMenuController()->setHitTestResult(result);
 
 #if ENABLE(INSPECTOR)
@@ -937,7 +931,7 @@ QWebHitTestResultPrivate* QWebPageAdapter::updatePositionDependentMenuActions(co
     WebCore::ContextMenu* webcoreMenu = page->contextMenuController()->contextMenu();
     QList<MenuItem> itemDescriptions;
     if (client && webcoreMenu)
-        itemDescriptions = descriptionForPlatformMenu(webcoreMenu->platformDescription(), page);
+        itemDescriptions = descriptionForPlatformMenu(webcoreMenu->items(), page);
     createAndSetCurrentContextMenu(itemDescriptions, visitedWebActions);
     if (result.scrollbar())
         return 0;
@@ -1026,7 +1020,7 @@ void QWebPageAdapter::didCloseInspector()
 void QWebPageAdapter::updateActionInternal(QWebPageAdapter::MenuAction action, const char* commandName, bool* enabled, bool* checked)
 {
     WebCore::FrameLoader* loader = mainFrameAdapter()->frame->loader();
-    WebCore::Editor* editor = page->focusController()->focusedOrMainFrame()->editor();
+    WebCore::Editor& editor = page->focusController()->focusedOrMainFrame()->editor();
 
     switch (action) {
     case QWebPageAdapter::Back:
@@ -1044,14 +1038,14 @@ void QWebPageAdapter::updateActionInternal(QWebPageAdapter::MenuAction action, c
     case QWebPageAdapter::SetTextDirectionDefault:
     case QWebPageAdapter::SetTextDirectionLeftToRight:
     case QWebPageAdapter::SetTextDirectionRightToLeft:
-        *enabled = editor->canEdit();
+        *enabled = editor.canEdit();
         *checked = false;
         break;
     default: {
 
         // if it's an editor command, let its logic determine state
         if (commandName) {
-            Editor::Command command = editor->command(commandName);
+            Editor::Command command = editor.command(commandName);
             *enabled = command.isEnabled();
             if (*enabled)
                 *checked = command.state() != FalseTriState;
@@ -1063,12 +1057,27 @@ void QWebPageAdapter::updateActionInternal(QWebPageAdapter::MenuAction action, c
     }
 }
 
+#if ENABLE(VIDEO)
+static WebCore::HTMLMediaElement* mediaElement(WebCore::Node* innerNonSharedNode)
+{
+    if (!(innerNonSharedNode && innerNonSharedNode->document()))
+        return 0;
+
+    if (!(innerNonSharedNode->renderer() && innerNonSharedNode->renderer()->isMedia()))
+        return 0;
+
+    if (innerNonSharedNode->hasTagName(WebCore::HTMLNames::videoTag) || innerNonSharedNode->hasTagName(WebCore::HTMLNames::audioTag))
+        return WebCore::toHTMLMediaElement(innerNonSharedNode);
+    return 0;
+}
+#endif
+
 void QWebPageAdapter::triggerAction(QWebPageAdapter::MenuAction action, QWebHitTestResultPrivate* hitTestResult, const char* commandName, bool endToEndReload)
 {
     Frame* frame = page->focusController()->focusedOrMainFrame();
     if (!frame)
         return;
-    Editor* editor = frame->editor();
+    Editor& editor = frame->editor();
 
     // Convenience
     QWebHitTestResultPrivate hitTest;
@@ -1099,10 +1108,10 @@ void QWebPageAdapter::triggerAction(QWebPageAdapter::MenuAction action, QWebHitT
 #if defined(Q_WS_X11)
         bool oldSelectionMode = Pasteboard::generalPasteboard()->isSelectionMode();
         Pasteboard::generalPasteboard()->setSelectionMode(true);
-        editor->copyURL(hitTestResult->linkUrl, hitTestResult->linkText);
+        editor.copyURL(hitTestResult->linkUrl, hitTestResult->linkText);
         Pasteboard::generalPasteboard()->setSelectionMode(oldSelectionMode);
 #endif
-        editor->copyURL(hitTestResult->linkUrl, hitTestResult->linkText);
+        editor.copyURL(hitTestResult->linkUrl, hitTestResult->linkText);
         break;
     }
     case OpenImageInNewWindow:
@@ -1114,6 +1123,9 @@ void QWebPageAdapter::triggerAction(QWebPageAdapter::MenuAction action, QWebHitT
     case DownloadLinkToDisk:
         frame->loader()->client()->startDownload(WebCore::ResourceRequest(hitTestResult->linkUrl, frame->loader()->outgoingReferrer()));
         break;
+    case DownloadMediaToDisk:
+        frame->loader()->client()->startDownload(WebCore::ResourceRequest(hitTestResult->mediaUrl, frame->loader()->outgoingReferrer()));
+        break;
     case Back:
         page->goBack();
         break;
@@ -1129,14 +1141,39 @@ void QWebPageAdapter::triggerAction(QWebPageAdapter::MenuAction action, QWebHitT
         break;
 
     case SetTextDirectionDefault:
-        editor->setBaseWritingDirection(NaturalWritingDirection);
+        editor.setBaseWritingDirection(NaturalWritingDirection);
         break;
     case SetTextDirectionLeftToRight:
-        editor->setBaseWritingDirection(LeftToRightWritingDirection);
+        editor.setBaseWritingDirection(LeftToRightWritingDirection);
         break;
     case SetTextDirectionRightToLeft:
-        editor->setBaseWritingDirection(RightToLeftWritingDirection);
+        editor.setBaseWritingDirection(RightToLeftWritingDirection);
         break;
+#if ENABLE(VIDEO)
+    case ToggleMediaControls:
+        if (HTMLMediaElement* mediaElt = mediaElement(hitTestResult->innerNonSharedNode))
+            mediaElt->setControls(!mediaElt->controls());
+        break;
+    case ToggleMediaLoop:
+        if (HTMLMediaElement* mediaElt = mediaElement(hitTestResult->innerNonSharedNode))
+            mediaElt->setLoop(!mediaElt->loop());
+        break;
+    case ToggleMediaPlayPause:
+        if (HTMLMediaElement* mediaElt = mediaElement(hitTestResult->innerNonSharedNode))
+            mediaElt->togglePlayState();
+    case ToggleMediaMute:
+        if (HTMLMediaElement* mediaElt = mediaElement(hitTestResult->innerNonSharedNode))
+            mediaElt->setMuted(!mediaElt->muted());
+        break;
+    case ToggleVideoFullscreen:
+        if (HTMLMediaElement* mediaElt = mediaElement(hitTestResult->innerNonSharedNode)) {
+            if (mediaElt->isVideo() && mediaElt->supportsFullscreen()) {
+                UserGestureIndicator indicator(DefinitelyProcessingUserGesture);
+                mediaElt->toggleFullscreenState();
+            }
+        }
+        break;
+#endif
 #if ENABLE(INSPECTOR)
     case InspectElement: {
         ASSERT(hitTestResult != &hitTest);
@@ -1146,7 +1183,7 @@ void QWebPageAdapter::triggerAction(QWebPageAdapter::MenuAction action, QWebHitT
 #endif
     default:
         if (commandName)
-            editor->command(commandName).execute();
+            editor.command(commandName).execute();
         break;
     }
 }
@@ -1215,6 +1252,23 @@ QString QWebPageAdapter::contextMenuItemTagForAction(QWebPageAdapter::MenuAction
     case ToggleUnderline:
         *checkable = true;
         return contextMenuItemTagUnderline();
+    case DownloadMediaToDisk:
+        return contextMenuItemTagDownloadMediaToDisk();
+    case CopyMediaUrlToClipboard:
+        return contextMenuItemTagCopyMediaLinkToClipboard();
+    case ToggleMediaControls:
+        *checkable = true;
+        return contextMenuItemTagShowMediaControls();
+    case ToggleMediaLoop:
+        *checkable = true;
+        return contextMenuItemTagToggleMediaLoop();
+    case ToggleMediaPlayPause:
+        return contextMenuItemTagMediaPlayPause();
+    case ToggleMediaMute:
+        *checkable = true;
+        return contextMenuItemTagMediaMute();
+    case ToggleVideoFullscreen:
+        return contextMenuItemTagToggleVideoFullscreen();
 
 #if ENABLE(INSPECTOR)
     case InspectElement:
@@ -1250,7 +1304,7 @@ void QWebPageAdapter::setSystemTrayIcon(QObject *icon)
 #endif // QT_NO_SYSTEMTRAYICON
 #endif // ENABLE(NOTIFICATIONS) || ENABLE(LEGACY_NOTIFICATIONS)
 
-#if ENABLE(GEOLOCATION)
+#if ENABLE(GEOLOCATION) && HAVE(QTPOSITIONING)
 void QWebPageAdapter::setGeolocationEnabledForFrame(QWebFrameAdapter* frame, bool on)
 {
     GeolocationPermissionClientQt::geolocationPermissionClient()->setPermission(frame, on);
@@ -1280,7 +1334,7 @@ bool QWebPageAdapter::hasFocusedNode() const
     Frame* frame = page->focusController()->focusedFrame();
     if (frame) {
         Document* document = frame->document();
-        hasFocus = document && document->focusedNode();
+        hasFocus = document && document->focusedElement();
     }
     return hasFocus;
 }
@@ -1312,7 +1366,7 @@ QWebPageAdapter::ViewportAttributes QWebPageAdapter::viewportAttributesForSize(c
 bool QWebPageAdapter::handleKeyEvent(QKeyEvent *ev)
 {
     Frame* frame = page->focusController()->focusedOrMainFrame();
-    return frame->eventHandler()->keyEvent(ev);
+    return frame->eventHandler()->keyEvent(PlatformKeyboardEvent(ev, m_useNativeVirtualKeyAsDOMKey));
 }
 
 bool QWebPageAdapter::handleScrolling(QKeyEvent *ev)
@@ -1322,10 +1376,10 @@ bool QWebPageAdapter::handleScrolling(QKeyEvent *ev)
     WebCore::ScrollGranularity granularity;
 
 #ifndef QT_NO_SHORTCUT
-    if (ev == QKeySequence::MoveToNextPage || (ev->key() == Qt::Key_Space && !(ev->modifiers() & Qt::ShiftModifier))) {
+    if (ev == QKeySequence::MoveToNextPage) {
         granularity = WebCore::ScrollByPage;
         direction = WebCore::ScrollDown;
-    } else if (ev == QKeySequence::MoveToPreviousPage || ((ev->key() == Qt::Key_Space) && (ev->modifiers() & Qt::ShiftModifier))) {
+    } else if (ev == QKeySequence::MoveToPreviousPage) {
         granularity = WebCore::ScrollByPage;
         direction = WebCore::ScrollUp;
     } else
@@ -1385,8 +1439,8 @@ void QWebPageAdapter::focusOutEvent(QFocusEvent *)
 bool QWebPageAdapter::handleShortcutOverrideEvent(QKeyEvent* event)
 {
     WebCore::Frame* frame = page->focusController()->focusedOrMainFrame();
-    WebCore::Editor* editor = frame->editor();
-    if (!editor->canEdit())
+    WebCore::Editor& editor = frame->editor();
+    if (!editor.canEdit())
         return false;
     if (event->modifiers() == Qt::NoModifier
         || event->modifiers() == Qt::ShiftModifier
@@ -1419,7 +1473,12 @@ bool QWebPageAdapter::touchEvent(QTouchEvent* event)
 {
 #if ENABLE(TOUCH_EVENTS)
     Frame* frame = mainFrameAdapter()->frame;
-    if (!frame->view())
+    if (!frame->view() || !frame->document())
+        return false;
+
+    // If the document doesn't have touch-event handles, we just ignore it
+    // and let QGuiApplication convert it to a mouse event.
+    if (!frame->document()->hasTouchEventHandlers())
         return false;
 
     // Always accept the QTouchEvent so that we'll receive also TouchUpdate and TouchEnd events
